# -*- coding: utf-8 -*-

"""audio - Manages sounds and music in the game."""

##Python imports##
from random import choice
##Panda3D imports##
from direct.showbase.DirectObject import DirectObject
##Local imports##
from .engine import Engine


class Audio(object):

    eventHandler = DirectObject()
    soundNames = ('button_click', 'breathing', 'footsteps', 'rock_scrape',
                  'door_close', 'door_open')
    soundExtension = '.ogg'
    sounds = {}
    soundEvents = {}
    currentSounds = []
    soundVolume = 1.0
    musicNames = ('Menu Music', 'Amoris Immortalis')
    musicExtension = '.ogg'
    music = {}
    blacklist = []
    currentTrack = None
    musicVolume = 0.75

    @classmethod
    def setMusicVolume(cls, volume):
        assert volume >= 0.0 and volume <= 1.0
        cls.musicVolume = volume

    @classmethod
    def setSoundVolume(cls, volume):
        assert volume >= 0.0 and volume <= 1.0
        cls.soundVolume = volume

    @classmethod
    def loadSounds(cls):
        for name in cls.soundNames:
            sound = Engine.base.loader.loadSfx(name + cls.soundExtension)
            cls.sounds[name] = sound

    @classmethod
    def loadMusic(cls):
        for name in cls.musicNames:
            musicFile = name.lower().replace(' ', '_') + cls.soundExtension
            track = Engine.base.loader.loadMusic(musicFile)
            cls.music[name] = track

    @classmethod
    def blacklistTrack(cls, exclusion):
        assert exclusion in cls.music
        if not exclusion in cls.blacklist:
            cls.blacklist.append(exclusion)

    @classmethod
    def playTrack(cls, track, loop=False, volume=None):
        if volume is None:
            volume = Audio.musicVolume
        assert track in cls.music
        if isinstance(loop, int):
            if not loop:
                cls.music[track].setLoop(False)
            else:
                cls.music[track].setLoopCount(loop)
        elif isinstance(loop, bool):
            cls.music[track].setLoop(loop)
        if volume is not cls.musicVolume:
            assert isinstance(volume, int) or isinstance(volume, float)
        cls.music[track].setVolume(volume)
        cls.music[track].play()
        cls.currentTrack = track

    @classmethod
    def stopTrack(cls):
        if not (cls.currentTrack and cls.music[cls.currentTrack].status() == 2):
            print "WARNING! The following did NOT pass:"
            print "cls.currentTrack and cls.music[cls.currentTrack].status() == 2"
            print "This may very well mean that the audio is broken!"
        cls.music[cls.currentTrack].stop()
        cls.currentTrack = None

    @classmethod
    def playRandomTrack(cls, loop=False, volume=None):
        if volume is None:
            volume = cls.musicVolume
        randomTrack = cls.selectRandomTrack()
        cls.playTrack(randomTrack, loop, volume)

    @classmethod
    def selectRandomTrack(cls):
        randomTrack = choice(list(cls.music.keys()))
        if randomTrack in cls.blacklist:
            randomTrack = cls.selectRandomTrack()
        return randomTrack

    @classmethod
    def addSoundEvent(cls, event, sound, volume=None, loop=False, playRate=1.0):
        if volume is None:
            volume = cls.soundVolume
        assert isinstance(event, str)
        assert sound in cls.sounds or sound is None
        if not event in cls.soundEvents:
            cls.soundEvents[event] = []
        assert isinstance(cls.soundEvents[event], list)
        cls.soundEvents[event].append(sound)
        cls.eventHandler.accept(event, cls.playSound, [sound, volume, loop,
                                                       playRate])

    @classmethod
    def playSound(cls, sound, volume=None, loop=False, playRate=1.0):
        if volume is None:
            volume = cls.soundVolume
        assert sound in cls.sounds
        if isinstance(loop, int):
            if not loop:
                cls.sounds[sound].setLoop(False)
            else:
                cls.sounds[sound].setLoop(loop)
        elif isinstance(loop, bool):
            cls.sounds[sound].setLoop(loop)
        if volume is not cls.soundVolume:
            assert isinstance(volume, int) or isinstance(volume, float)
        cls.sounds[sound].setVolume(volume)
        assert isinstance(playRate, int) or isinstance(playRate, float)
        cls.sounds[sound].setPlayRate(playRate)
        cls.sounds[sound].setFinishedEvent('finished' + sound)
        cls.eventHandler.accept('finished' + sound, cls.stopSound, [sound])
        if cls.sounds[sound].status() != 2:
            cls.sounds[sound].play()
            cls.currentSounds.append(sound)

    @classmethod
    def stopSound(cls, sound):
        assert sound in cls.currentSounds
        cls.sounds[sound].stop()
        cls.currentSounds.remove(sound)

    @classmethod
    def stopAllSounds(cls):
        for sound in cls.currentSounds:
            cls.sounds[sound].stop()
            cls.currentSound.remove(sound)

