# -*- coding: utf-8 -*-

"""engine - Module that handles window and ShowBase stuff."""

##Python imports##
from sys import platform, exit

##Panda3D imports##
from direct.showbase.ShowBase import ShowBase
from panda3d.core import loadPrcFile, loadPrcFileData, ExecutionEnvironment, Filename
from panda3d.core import WindowProperties, ModifierButtons, CollisionTraverser
from panda3d.core import CollisionHandlerPusher, CollisionHandlerGravity
import direct.gui.DirectGuiGlobals as DGG

DGG.setDefaultClickSound(None)
DGG.setDefaultRolloverSound(None)

class Engine(object):

    rootDirectory = ExecutionEnvironment.getEnvironmentVariable("MAIN_DIR")
    # We're using the ordinary Pusher because the
    # FluidPusher gives weird wall-humping behaviour.
    pusher = CollisionHandlerPusher()
    pusher.setHorizontal(True)
    gravity = CollisionHandlerGravity()
    gravity.setGravity(100)
    gravity.setReach(3)
    gravity.setOffset(1)

    @classmethod
    def start(cls):
        cls.config = loadPrcFile(Filename(Engine.rootDirectory, 'config.prc'))
        if platform in ["win32", "darwin"]:
            loadPrcFileData("", "audio-library-name p3fmod_audio")
        else:
            loadPrcFileData("", "audio-library-name p3openal_audio")

        # Initialise ShowBase to open the window.
        DGG.setDefaultClickSound(None)
        DGG.setDefaultRolloverSound(None)
        cls.base = ShowBase()
        cls.base.disableMouse()
        cls.base.camLens.setFov(70)
        cls.base.pushCTrav(CollisionTraverser())
        cls.base.mouseWatcherNode.setModifierButtons(ModifierButtons())
        cls.base.buttonThrowers[0].node().setModifierButtons(ModifierButtons())
        cls.base.accept('escape', cls.exit)
        cls.fonts = {
            'Ink Calligraphy': cls.base.loader.loadFont('Ink_Calligraphy.ttf'),
            'Promocyja': cls.base.loader.loadFont('Promocyja.ttf')}
        cls.globalClock = globalClock

    @classmethod
    def toggleCursor(cls, override=None):
        curProps = cls.base.win.getProperties()
        currentSetting = curProps.getCursorHidden()
        if override is not None:
            newSetting = override
        else:
            newSetting = not currentSetting
        if newSetting is not currentSetting:
            newProps = WindowProperties()
            newProps.setCursorHidden(not newSetting)
            if newSetting:
                newProps.setMouseMode(WindowProperties.MAbsolute)
            else:
                newProps.setMouseMode(WindowProperties.MRelative)
                newProps.setForeground(True)
            cls.base.win.requestProperties(newProps)
            if not newSetting:
                cls.centreCursor()

    @classmethod
    def toggleFullscreen(cls, override=None):
        curProps = cls.base.win.getProperties()
        currentSetting = curProps.getFullscreen()
        if override is not None:
            newSetting = override
        else:
            newSetting = not currentSetting
        if newSetting is not currentSetting:
            newProps = WindowProperties()
            newProps.setFullscreen(newSetting)
            cls.base.win.requestProperties(newProps)

    @classmethod
    def centreCursor(cls):
        cls.windowCentreX = cls.base.win.getXSize() / 2
        cls.windowCentreY = cls.base.win.getYSize() / 2
        return cls.base.win.movePointer(0, cls.windowCentreX, cls.windowCentreY)

    @classmethod
    def exit(cls):
        exit()

