# -*- coding: utf-8 -*-

"""mainmenu - Menu that appears at the start of the game."""

##Panda3D imports##
from direct.gui.DirectGui import DGG, DirectFrame, DirectLabel, DirectButton
from direct.gui.DirectGui import DirectSlider
from direct.fsm.FSM import FSM
from direct.gui.OnscreenText import OnscreenText
from direct.gui.OnscreenImage import OnscreenImage

from .engine import Engine
from .audio import Audio


class MainMenu(object, FSM):

    def __init__(self):
        FSM.__init__(self, 'MainMenuFSM')
        Engine.base.setBackgroundColor(0, 0, 0, 1)
        # Main frame: Start Game, Settings, Exit
        self.mainFrame = DirectFrame(frameColor=(0.26, 0.18, 0.06, 1.0),
                                     frameSize=(-1, 1, -0.75, 0.75),
                                     pos=(0, -1, 0), relief=DGG.RIDGE,
                                     borderWidth=(0.11, 0.1))
        self.title = DirectLabel(text='Enigmatic Temple\nRoom Escape', 
                                 relief=None,
                                 frameSize=(-0.5, 0.5, -0.1, 0.1),
                                 text_scale=(0.2, 0.2), pos=(0, 0, 0.45),
                                 text_font=Engine.fonts['Promocyja'],
                                 text_fg=(0.79, 0.69, 0.57, 1))
        self.title.reparentTo(self.mainFrame)
        self.startGame = DirectButton(text='Start Game', relief=None,
                                      frameSize=(-0.2, 0.2, -0.05, 0.05),
                                      pos=(0, 0, -0.25), text_scale=(0.15, 0.15),
                                      text_pos=(0, -0.01),
                                      text_font=Engine.fonts['Ink Calligraphy'],
                                      text_fg=(0.79, 0.69, 0.57, 1),
                                      clickSound=Audio.sounds['button_click'],
                                      command=self.gameStart)
        self.startGame.reparentTo(self.mainFrame)
        #self.settings = DirectButton(text='Settings', relief=None,
        #                             frameSize=(-0.2, 0.2, -0.05, 0.05),
        #                             pos=(0, 0, -0.25), text_scale=(0.15, 0.15),
        #                             text_font=Engine.fonts['Ink Calligraphy'],
        #                             text_fg=(0.79, 0.69, 0.57, 1),
        #                             clickSound=Audio.sounds['button_click'],
        #                             command=self.request,
        #                             extraArgs=['Settings'])
        #self.settings.reparentTo(self.mainFrame)
        self.exitGame = DirectButton(text='Exit', relief=None,
                                     frameSize=(-0.2, 0.2, -0.05, 0.05),
                                     pos=(0, 0, -0.5), text_scale=(0.15, 0.15),
                                     text_font=Engine.fonts['Ink Calligraphy'],
                                     text_fg=(0.79, 0.69, 0.57, 1),
                                     clickSound=Audio.sounds['button_click'],
                                     command=Engine.exit)
        self.exitGame.reparentTo(self.mainFrame)

        # Settings frame: Fullscreen, Music Volume, Sound Volume
        self.settingsFrame = DirectFrame(frameColor=(0.26, 0.18, 0.06, 1.0),
                                         frameSize=(-1, 1, -0.75, 0.75),
                                         pos=(0, -1, 0), relief=DGG.RIDGE,
                                         borderWidth=(0.11, 0.1))
        self.fullscreenLabel = DirectLabel()
        self.fullscreenLabel.reparentTo(self.settingsFrame)
        self.fullscreen = DirectButton(text='Off', relief=None,
                                       frameSize=(-0.2, 0.2, -0.05, 0.05),
                                       pos=(0.7, 0, 0), text_scale=(0.15, 0.15),
                                       text_font=Engine.fonts['Ink Calligraphy'],
                                       text_fg=(0.79, 0.69, 0.57, 1),
                                       clickSound=Audio.sounds['button_click'],
                                       command=self.setFullscreen)
        self.fullscreen.reparentTo(self.settingsFrame)
        self.musicLabel = DirectLabel()
        self.musicLabel.reparentTo(self.settingsFrame)
        self.musicVolume = DirectSlider(range=(0,100), value=50, pageSize=3,
                                        frameColor=(0.54, 0.37, 0.18, 1),
                                        frameSize=(-0.2, 0.2, -0.05, 0.05),
                                        pos=(0, 0, 0.1),
                                        command=self.setVolume,
                                        extraArgs=['music'])
        self.musicVolume.reparentTo(self.settingsFrame)
        self.toMain = DirectButton(text='Back', relief=None,
                                   frameSize=(-0.2, 0.2, -0.05, 0.05),
                                   pos=(0, 0, -0.5), text_scale=(0.15, 0.15),
                                   text_font=Engine.fonts['Ink Calligraphy'],
                                   text_fg=(0.79, 0.69, 0.57, 1),
                                   clickSound=Audio.sounds['button_click'],
                                   command=self.request, extraArgs=['Main'])
        self.toMain.reparentTo(self.settingsFrame)

        self.panda3dLogo = OnscreenImage(image='panda3d_logo.png',
                                         pos=(1.175, 0, -0.84), scale=0.125)
        self.panda3dLogo.setTransparency(1)

        self.request('Main')
        Engine.base.accept('y', self.gameStart)

    def setVolume(self, type):
        if type == 'music':
            Audio.setMusicVolume(self.musicVolume['value'] / 100.0)
        elif type == 'sound':
            Audio.setSoundVolume(self.soundVolume['value'] / 100.0)

    def setFullscreen(self):
        Engine.toggleFullscreen()
        if self.fullscreen['text'] == 'Off':
            self.fullscreen['text'] = 'On'
        else:
            self.fullscreen['text'] = 'Off'

    def gameStart(self):
        font = Engine.fonts['Ink Calligraphy']
        instructionText = OnscreenText('Loading...', mayChange=True,
                                       pos=(0, -0.9), scale=0.1, font=font,
                                       fg=(1, 1, 1, 1), bg=(0, 0, 0, 0.5))
        instructionText.setTransparency(1)
        Engine.base.graphicsEngine.renderFrame()
        Engine.base.graphicsEngine.renderFrame()
        Engine.base.messenger.send('StartGame')
        instructionText.destroy()
        self.panda3dLogo.destroy()

    def destroy(self):
        self.mainFrame.destroy()
        self.settingsFrame.destroy()

    def enterMain(self):
        self.settingsFrame.hide()
        self.mainFrame.show()

    def enterSettings(self):
        self.mainFrame.hide()
        self.settingsFrame.show()

