# -*- coding: utf-8 -*-

"""controller - Gives the player control over Statue rotation."""

##Panda3D imports##
from direct.fsm.FSM import FSM
from direct.showbase.DirectObject import DirectObject
from panda3d.core import CardMaker, Point3, NodePath
##Local imports##
from src.engine import Engine


class Controller(object, FSM, DirectObject):

    def __init__(self, nodePath, platform):
        FSM.__init__(self, 'ControllerFSM')
        self.platform = platform
        self.node = nodePath
        self.node.setPythonTag('clickable', self)
        self.activeStatue = None
        self.acceptOnce('BalancingPuzzleVictory', self.terminate)

    def setupControls(self):
        self.accept('arrow_up', self.setActiveStatue, ['next'])
        self.accept('arrow_down', self.setActiveStatue, ['prev'])
        self.accept('arrow_left', self.platform.startRotation,
                    ['statue', 'left'])
        self.accept('arrow_left-up', self.platform.removeTask,
                    ['RotateStatueTask'])
        self.accept('arrow_right', self.platform.startRotation,
                    ['statue', 'right'])
        self.accept('arrow_right-up', self.platform.removeTask,
                    ['RotateStatueTask'])
        self.accept(',', self.platform.startRotation, ['platform', 'left'])
        self.accept(',-up', self.platform.removeTask, ['RotatePlatformTask'])
        self.accept('.', self.platform.startRotation, ['platform', 'right'])
        self.accept('.-up', self.platform.removeTask, ['RotatePlatformTask'])
        if self.activeStatue is None:
            self.setActiveStatue(0)

    def setActiveStatue(self, statue):
        if isinstance(statue, str):
            self.platform.statues[self.activeStatue].node.clearColorScale()
            if statue == 'next':
                self.activeStatue += 1
                if self.activeStatue > 3:
                    self.activeStatue = 0
            elif statue == 'prev':
                self.activeStatue -= 1
                if self.activeStatue < 0:
                    self.activeStatue = 3
        elif isinstance(statue, int):
            if statue >= 0 and statue <= 3:
                self.activeStatue = statue
        self.platform.activeStatue = self.activeStatue
        newStatue = self.platform.statues[self.activeStatue]
        newStatue.node.setColorScale(5, 5, 3, 0.5)

    def terminate(self):
        self.ignoreAll()
        self.platform.removeTask('RotatePlatformTask')
        self.platform.removeTask('RotateStatueTask')

