# -*- coding: utf-8 -*-

"""platform - Hemisphere that supports weights in the balancing puzzle."""

##Python imports##
from random import random, randint

##Panda3D imports##
from direct.showbase.DirectObject import DirectObject
from direct.interval.FunctionInterval import Func
from direct.interval.IntervalGlobal import Parallel
from panda3d.core import NodePath, Vec2, Vec3

##Local imports##
from src.engine import Engine
from src.audio import Audio


class Platform(object, DirectObject):

    def __init__(self, nodePath, numberOfStatues=4):
        self.subnode = nodePath
        self.node = nodePath.getParent()
        self.statues = []
        self.activeStatue = 0

        self.task = Engine.base.taskMgr.add(self._task, 'platform_task')

    def _task(self, task):
        leanVector = Vec2(0, 0)
        for s in self.statues:
            pos = s.node.getPos(self.subnode)
            vec = Vec2(pos[0], pos[1])
            leanVector += vec * s.weight
        self.node.lookAt(self.node.getParent(),
                         self.node.getPos() + Vec3(0, leanVector[1], 10000.0))
        # LOL terrible hack
        heading = self.node.getH()
        if heading == 180.0:
            self.node.setR(self.node.getR() + 180.0)
        elif heading == 0.0:
            pass
        else:
            assert False, "Terrible hack failed miserably with H = " + \
                          str(heading)
            pass
        #self.node.setH(0)
        return task.cont

    def shufflePositions(self, iterations=10):
        for i in range(iterations):
            statue = self.statues[randint(0, len(self.statues) - 1)]
            rotationAmount = random() * 180.0 + 90.0
            statue.rotate(rotationAmount, playSound=False)
        rotationAmount = random() * 180.0 + 90.0
        self.subnode.getChild(0).setH(rotationAmount)

    def startRotation(self, part, direction):
        if direction == 'left':
            if part == 'platform':
                self.removeTask('RotatePlatformTask')
                self.addTask(self._rotatePlatform, 'RotatePlatformTask',
                         extraArgs=[-30], appendTask=True)
            elif part == 'statue':
                self.removeTask('RotateStatueTask')
                self.addTask(self._rotateStatue, 'RotateStatueTask',
                             extraArgs=[-30], appendTask=True)
        elif direction == 'right':
            if part == 'platform':
                self.removeTask('RotatePlatformTask')
                self.addTask(self._rotatePlatform, 'RotatePlatformTask',
                         extraArgs=[30], appendTask=True)
            elif part == 'statue':
                self.removeTask('RotateStatueTask')
                self.addTask(self._rotateStatue, 'RotateStatueTask',
                             extraArgs=[30], appendTask=True)

    def _rotatePlatform(self, amount, task=None):
        dt = Engine.globalClock.getDt()
        amount *= dt
        rotationAmount = amount + self.subnode.getChild(0).getH()
        rotateInterval = Func(self.subnode.getChild(0).setH, rotationAmount)
        soundInterval = Func(Audio.playSound, 'rock_scrape', 0.7, playRate=0.5)
        rotate = Parallel(rotateInterval, soundInterval)
        rotate.start()
        if task:
            return task.cont

    def _rotateStatue(self, amount, task):
        dt = Engine.globalClock.getDt()
        self.statues[self.activeStatue].rotate(amount * dt)
        return task.cont

