# -*- coding: utf-8 -*-

"""statue - Object that moves across the platform in the balancing puzzle."""

##Python imports##
from random import random
##Panda3D imports##
from direct.interval.LerpInterval import LerpHprInterval
from direct.interval.FunctionInterval import Func
from direct.interval.IntervalGlobal import Parallel
from panda3d.core import NodePath
##Local imports##
from src.engine import Engine
from src.audio import Audio


class Statue(object):

    partners = {0: 2, 1: 0, 2: 3, 3: 1}
    statues = []
    models = ["sun_statue", "moon_statue", "lightning_statue", "fire_statue"]

    def __init__(self, number, nodePath, weight=10.0):
        self.number = number
        self.node = nodePath
        self.weight = weight
        self.rotationNode = NodePath('RotationNode')
        self.rotationNode.reparentTo(nodePath.getParent())
        self.node.reparentTo(self.rotationNode)

        self.model = Engine.base.loader.loadModel(self.models.pop(0))
        self.model.setScale(15)
        self.model.reparentTo(self.node)

        Statue.statues.append(self)

    def rotate(self, amount, playSound=True):
        rotationAmount = amount + self.rotationNode.getH()
        partner = Statue.statues[Statue.partners[self.number]]
        if self.number == 2 or self.number == 3:
            partnerAmount = (-3 * amount) + partner.rotationNode.getH()
        else:
            partnerAmount = (-2 * amount + partner.rotationNode.getH())
        rotateInterval = Func(self.rotationNode.setH, rotationAmount)
        partnerRotateInterval = Func(partner.rotationNode.setH, partnerAmount)
        if playSound:
            soundVolume = 0.8
        else:
            soundVolume = 0
        soundInterval = Func(Audio.playSound, 'rock_scrape', soundVolume)
        rotate = Parallel(rotateInterval, partnerRotateInterval, soundInterval)
        rotate.start()

